--
-- GrassRemoving
--
--
-- @author jodamm
-- @date 16/12/2018
--

GrassRemoving = {};

-- function to register specializations to vehicleTypes
function GrassRemoving.registerSpecialization(vehicleType, specialization, specName)
    table.insert(vehicleType.specializationNames, specName)
    vehicleType.specializationsByName[specName] = specialization
    table.insert(vehicleType.specializations, specialization)
end

-- register grassRemoving to vehicleType roller
if  g_vehicleTypeManager.vehicleTypes.roller ~= nil then
    GrassRemoving.registerSpecialization(g_vehicleTypeManager.vehicleTypes.roller, GrassRemoving, 'grassRemoving')
end

function GrassRemoving.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Roller, specializations)
end

function GrassRemoving:onLoad(savegame)
    local spec = self.spec_grassRemoving
    if savegame ~= nil then
        spec.removeGrassWG = Utils.getNoNil(getXMLBool(savegame.xmlFile, savegame.key..".grassRemoving#removeGrassWG"),false)
    else
        spec.removeGrassWG = false
    end

end

function GrassRemoving:saveToXMLFile(nodeIdent,path,a)
    local spec = self.spec_grassRemoving
    setXMLBool(nodeIdent, path .. '#removeGrassWG', spec.removeGrassWG)
end

function GrassRemoving.registerOverwrittenFunctions(vehicleType)
    vehicleType.functions.processRollerArea = Utils.overwrittenFunction(vehicleType.functions.processRollerArea, GrassRemoving.processRollerArea)
end

function GrassRemoving.registerFunctions(vehicleType)
    SpecializationUtil.registerFunction(vehicleType, "getWGRemoveGrass", GrassRemoving.getWGRemoveGrass)
end

function GrassRemoving.registerEventListeners(vehicleType)
    SpecializationUtil.registerEventListener(vehicleType, "onLoad", GrassRemoving)
    SpecializationUtil.registerEventListener(vehicleType, "onUpdate", GrassRemoving)
    SpecializationUtil.registerEventListener(vehicleType, "onRegisterActionEvents", GrassRemoving)
end

function GrassRemoving:onRegisterActionEvents(isActiveForInput, isOnActiveVehicle)
    if self.isClient then
        local spec = self.spec_grassRemoving
        self:clearActionEventsTable(spec.actionEvents)
        if self:getIsActiveForInput(true) then
            local actionEventId            
            _, actionEventId = self:addActionEvent(spec.actionEvents, 'WG_TOGGLE_REMOVE_GRASS', self, GrassRemoving.actionToggleRemoveGrass, false, true, false, true, nil)
            if isActiveForInput then
                g_inputBinding:setActionEventTextPriority(actionEventId, GS_PRIO_VERY_HIGH)
            elseif isOnActiveVehicle then
              g_inputBinding:setActionEventTextPriority(actionEventId, GS_PRIO_NORMAL)
            end
            g_inputBinding:setActionEventTextVisibility(actionEventId, false)
            spec.toggleRemoveGrassEventId = actionEventId 
        end
    end
end

function GrassRemoving:onUpdate(dt, isActiveForInput, isSelected)
    local spec = self.spec_grassRemoving
    if isActiveForInput then
        local fruitType = g_fruitTypeManager.nameToFruitType['GRASS']
        if fruitType ~= nil then
            g_inputBinding:setActionEventText(spec.toggleRemoveGrassEventId, g_i18n:getText("WG_remove_grass").." : ".. self:getWGRemoveGrass())
            g_inputBinding:setActionEventActive(spec.toggleRemoveGrassEventId, true)
            g_inputBinding:setActionEventTextVisibility(spec.toggleRemoveGrassEventId, true)
        end
    else
        g_inputBinding:setActionEventActive(spec.toggleRemoveGrassEventId, false)
        g_inputBinding:setActionEventTextVisibility(spec.toggleRemoveGrassEventId, false)
    end
end

function GrassRemoving.actionToggleRemoveGrass(self, actionName, inputValue, callbackState, isAnalog)
    local spec = self.spec_grassRemoving
    spec.removeGrassWG = not spec.removeGrassWG
end

function GrassRemoving:processRollerArea(superFunc, workArea, arg1)
    local spec = self.spec_grassRemoving
    local fruitType = g_fruitTypeManager.nameToFruitType['GRASS']
    if spec.removeGrassWG then
        if fruitType ~= nil then
            fruitType.destroyedByRoller = true
        end
    end
    local modArea = superFunc(self, workArea, arg1)
    if fruitType ~= nil then
        fruitType.destroyedByRoller = false
    end
    return  modArea
end;

function GrassRemoving:getWGRemoveGrass()
    local spec = self.spec_grassRemoving
    if spec.removeGrassWG == true then
        return g_i18n:getText("button_yes");
    else
        return g_i18n:getText("button_no");
    end
end
