--
-- GrassSowing
--
--
-- @author jodamm
-- @date 15/12/2018
--

GrassSowing = {};

-- function to register specializations to vehicleTypes
function GrassSowing.registerSpecialization(vehicleType, specialization, specName)
    table.insert(vehicleType.specializationNames, specName)
    vehicleType.specializationsByName[specName] = specialization
    table.insert(vehicleType.specializations, specialization)
end

-- register grassSowing to vehicleType sowingMachine
if  g_vehicleTypeManager.vehicleTypes.sowingMachine ~= nil then
    GrassSowing.registerSpecialization(g_vehicleTypeManager.vehicleTypes.sowingMachine, GrassSowing, 'grassSowing')
end

function GrassSowing.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(SowingMachine, specializations)
end

function GrassSowing:onLoad(savegame)
    local spec = self.spec_grassSowing
    if g_currentMission.fruits[1] ~= nil then
        local parent = getParent(g_currentMission.fruits[1].id)
        local numChilds = getNumOfChildren(parent)
        for i=0, numChilds-1 do
            childId = getChildAt(parent, i);
            if getName(childId) == 'decoFoliage' then
                GrassSowing.decoFoliageId = childId
            elseif getName(childId) == 'bushUS01' then
                GrassSowing.bushUS01Id = childId
            end
        end
    end 
    if g_currentMission.densityMapModifiers['updateGrassSowingArea'] == nil then
        local dMM = {}
        dMM.modifier = DensityMapModifier:new(g_currentMission.terrainDetailId,0,3)
        dMM.filter = DensityMapFilter:new(dMM.modifier)
        dMM.mask = DensityMapFilter:new(g_currentMission.terrainDetailId,0,3)
        dMM.decoFilter = DensityMapFilter:new(GrassSowing.decoFoliageId,0,4)
        g_currentMission.densityMapModifiers['updateGrassSowingArea'] = dMM
    end
    spec.lastStartWorldX = -4000
    spec.lastStartWorldZ = -4000
    if savegame ~= nil then
        spec.grassOutsideFields = Utils.getNoNil(getXMLBool(savegame.xmlFile, savegame.key..".grassSowing#grassOutsideFields"),false)
        spec.weedsOutsideFields = Utils.getNoNil(getXMLFloat(savegame.xmlFile, savegame.key..".grassSowing#weedsOutsideFields"),0)
    else
        spec.grassOutsideFields = false
        spec.weedsOutsideFields = 0
    end
end

function GrassSowing:saveToXMLFile(nodeIdent,path,a)
    local spec = self.spec_grassSowing
    setXMLBool(nodeIdent, path .. '#grassOutsideFields', spec.grassOutsideFields)
    setXMLFloat(nodeIdent, path .. '#weedsOutsideFields', spec.weedsOutsideFields)
end

function GrassSowing.initSpecialization(vehicleType)
end

function GrassSowing.registerOverwrittenFunctions(vehicleType)
    vehicleType.functions.processSowingMachineArea = Utils.overwrittenFunction(vehicleType.functions.processSowingMachineArea, GrassSowing.processSowingMachineArea)
end

function GrassSowing.registerFunctions(vehicleType)
    SpecializationUtil.registerFunction(vehicleType, "getGrassOutsideFields", GrassSowing.getGrassOutsideFields)
    SpecializationUtil.registerFunction(vehicleType, "getWeedsOutsideFields", GrassSowing.getWeedsOutsideFields)
end

function GrassSowing.registerEventListeners(vehicleType)
    SpecializationUtil.registerEventListener(vehicleType, "onLoad", GrassSowing)
    SpecializationUtil.registerEventListener(vehicleType, "onUpdate", GrassSowing)
    SpecializationUtil.registerEventListener(vehicleType, "onRegisterActionEvents", GrassSowing)
end

function GrassSowing:onRegisterActionEvents(isActiveForInput, isOnActiveVehicle)
    if self.isClient then
        local spec = self.spec_grassSowing
        self:clearActionEventsTable(spec.actionEvents)
        if self:getIsActiveForInput(true) then
            local actionEventId            
            _, actionEventId = self:addActionEvent(spec.actionEvents, 'WG_TOGGLE_WEEDS', self, GrassSowing.actionToggleWeeds, false, true, false, true, nil)
            if isActiveForInput then
                g_inputBinding:setActionEventTextPriority(actionEventId, GS_PRIO_VERY_HIGH)
            elseif isOnActiveVehicle then
              g_inputBinding:setActionEventTextPriority(actionEventId, GS_PRIO_NORMAL)
            end
            g_inputBinding:setActionEventTextVisibility(actionEventId, false)
            spec.toggleWeedsEventId = actionEventId 
            
            _, actionEventId = self:addActionEvent(spec.actionEvents, 'WG_TOGGLE_GRASS', self, GrassSowing.actionToggleGrass, false, true, false, true, nil)
            if isActiveForInput then
                g_inputBinding:setActionEventTextPriority(actionEventId, GS_PRIO_VERY_HIGH)
            elseif isOnActiveVehicle then
              g_inputBinding:setActionEventTextPriority(actionEventId, GS_PRIO_NORMAL)
            end
            g_inputBinding:setActionEventTextVisibility(actionEventId, false)
            spec.toggleGrassEventId = actionEventId           
        end
    end
end

function GrassSowing:onUpdate(dt, isActiveForInput, isSelected)
    local spec = self.spec_grassSowing
    
    if isActiveForInput then
        local fruitId = self.spec_sowingMachine.seeds[self.spec_sowingMachine.currentSeed]
        local fruitType = g_currentMission.fruitTypeManager.indexToFruitType[fruitId]
        if fruitType ~= nil and fruitType.name == 'GRASS' then
            g_inputBinding:setActionEventText(spec.toggleGrassEventId, g_i18n:getText("WG_grass_outside_fields").." : ".. self:getGrassOutsideFields())
            g_inputBinding:setActionEventActive(spec.toggleGrassEventId, true)
            g_inputBinding:setActionEventTextVisibility(spec.toggleGrassEventId, true)
            if spec.grassOutsideFields then
                g_inputBinding:setActionEventText(spec.toggleWeedsEventId, g_i18n:getText("WG_weeds_outside_fields").." : ".. self:getWeedsOutsideFields())
                g_inputBinding:setActionEventActive(spec.toggleWeedsEventId, true)
                g_inputBinding:setActionEventTextVisibility(spec.toggleWeedsEventId, true)
            else
                g_inputBinding:setActionEventActive(spec.toggleWeedsEventId, false)
                g_inputBinding:setActionEventTextVisibility(spec.toggleWeedsEventId, false)          
            end        
        else
            g_inputBinding:setActionEventActive(spec.toggleGrassEventId, false)
            g_inputBinding:setActionEventTextVisibility(spec.toggleGrassEventId, false)        

            g_inputBinding:setActionEventActive(spec.toggleWeedsEventId, false)
            g_inputBinding:setActionEventTextVisibility(spec.toggleWeedsEventId, false)          
        end
    end
end

function GrassSowing.actionToggleGrass(self, actionName, inputValue, callbackState, isAnalog)
    local spec = self.spec_grassSowing
    spec.grassOutsideFields = not spec.grassOutsideFields
end

function GrassSowing.actionToggleWeeds(self, actionName, inputValue, callbackState, isAnalog)
    local spec = self.spec_grassSowing
    spec.weedsOutsideFields = spec.weedsOutsideFields + 10
    if spec.weedsOutsideFields > 50 then
        spec.weedsOutsideFields = 0
    end    
end

function GrassSowing:processSowingMachineArea(superFunc, workArea, angle)
    local modArea, area = superFunc(self, workArea, angle)
    local spec = self.spec_grassSowing
    local fruitId = self.spec_sowingMachine.seeds[self.spec_sowingMachine.currentSeed]
    local fruitType = g_currentMission.fruitTypeManager.indexToFruitType[fruitId]
    if fruitType ~= nil and fruitType.name == 'GRASS' and spec.grassOutsideFields then
        local x1,y1,z1 = getWorldTranslation(workArea.start)
				local x2,y2,z2 = getWorldTranslation(workArea.width)
				local x3,y3,z3 = getWorldTranslation(workArea.height)
        local diffX1 = math.abs(self.spec_grassSowing.lastStartWorldX - x1)
        local diffZ1 = math.abs(self.spec_grassSowing.lastStartWorldZ - z1) 
        if diffX1 > 0.7 or diffZ1 > 0.7 then
            local areaValueB, modAreaB, areaB = GrassSowing.updateGrassSowingArea(self,fruitId, x1, z1, x2, z2, x3, z3, angle, 2)       
            modArea = modArea + modAreaB
            self.spec_sowingMachine.workAreaParameters.lastChangedArea = modArea
        end
    end
    return modArea, area
end

function GrassSowing:updateGrassSowingArea(fruitId, startWorldX, startWorldZ, widthWorldX, widthWorldZ, heightWorldX, heightWorldZ, angle, growState)
    local spec = self.spec_grassSowing
    local ids = g_currentMission.fruits[fruitId];
    if ids == nil or ids.id == 0 then
        return 0;
    end
    local detailId = g_currentMission.terrainDetailId;
    local mapModifier = g_currentMission.densityMapModifiers.updateGrassSowingArea.modifier
    local mapFilter = g_currentMission.densityMapModifiers.updateGrassSowingArea.filter
    local mapMask = g_currentMission.densityMapModifiers.updateGrassSowingArea.mask

    mapModifier:setParallelogramWorldCoords( startWorldX, startWorldZ, widthWorldX, widthWorldZ, heightWorldX, heightWorldZ, 'ppp')
    mapModifier:resetDensityMapAndChannels(ids.id,0,4) --todo Variable suchen
    mapFilter:resetDensityMapAndChannels(ids.id,0,4) --todo Variable suchen

    mapMask:setValueCompareParams('equal', 0) -- Ausserhalb vom Feld
    mapFilter:setValueCompareParams('equal', 0) -- kein Gras

    spec.lastStartWorldX = startWorldX       
    spec.lastStartWorldZ = startWorldZ
    local areaValue, modArea, area = mapModifier:executeSet(growState,mapMask,mapFilter) --Wachstumsstufe sen
    if modArea > 0 and spec.weedsOutsideFields > 0 and GrassSowing.decoFoliageId ~= nil then        
        GrassSowing.addRandomWeedArea(startWorldX, startWorldZ, widthWorldX, widthWorldZ, heightWorldX, heightWorldZ, spec.weedsOutsideFields)
    end
    return areaValue, modArea, area
end 

function GrassSowing.addRandomWeedArea(startWorldX, startWorldZ, widthWorldX, widthWorldZ, heightWorldX, heightWorldZ, weedPercent)
    if Utils.randomFloat(100,0) < weedPercent then
        local diffX = startWorldX - widthWorldX
        local diffZ = startWorldZ - widthWorldZ
        local rndStartX = Utils.randomFloat(math.max(diffX, 0), math.min(diffX, 0))
        diffX = diffX - rndStartX
        local rndEndX = Utils.randomFloat(math.max(diffX, 0), math.min(diffX, 0))
        local rndStartZ = Utils.randomFloat(math.max(diffZ, 0), math.min(diffZ, 0))
        diffZ = diffZ - rndStartZ 
        local rndEndZ = Utils.randomFloat(math.max(diffZ, 0), math.min(diffZ, 0))
        local x1 = startWorldX - rndStartX
        local x2 = widthWorldX + rndEndX
        local x3 = heightWorldX
        local z1 = startWorldZ - rndStartZ
        local z2 = widthWorldZ + rndEndZ
        local z3 = heightWorldZ
        local mapModifier = g_currentMission.densityMapModifiers.updateGrassSowingArea.modifier
        local mapFilter = g_currentMission.densityMapModifiers.updateGrassSowingArea.filter
        local mapMask = g_currentMission.densityMapModifiers.updateGrassSowingArea.mask
        mapModifier:setParallelogramWorldCoords( x1, z1, x2, z2, x3, z3, 'ppp')
        mapModifier:resetDensityMapAndChannels(GrassSowing.decoFoliageId,0,4) --todo Variable suchen
        mapFilter:resetDensityMapAndChannels(GrassSowing.decoFoliageId,0,4) --todo Variable suchen
        mapMask:setValueCompareParams('equal', 0) -- Ausserhalb vom Feld
        mapFilter:setValueCompareParams('equal', 0) -- kein Deco
        mapModifier:executeSet(Utils.randomFloat(10,0),mapMask,mapFilter) --deco sen
    end
end

function GrassSowing:getGrassOutsideFields()
    local spec = self.spec_grassSowing
    if spec.grassOutsideFields == true then
        return g_i18n:getText("button_yes");
    else
        return g_i18n:getText("button_no");
    end
end

function GrassSowing:getWeedsOutsideFields()
    local spec = self.spec_grassSowing
    if spec.weedsOutsideFields > 0 then
        return spec.weedsOutsideFields .. " %" 
    else
        return g_i18n:getText("button_no")
    end
end


